package de.tu_darmstadt.tk.SmartHomeNetworkSim.view;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowEvent;
import java.awt.event.WindowStateListener;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import java.awt.BorderLayout;

import javax.swing.SwingConstants;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.util.Utility;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups.SimulationConfigurator;

/**
 * Main class of the GUI, which combines the UserInterface and visualisation of the simulation.
 *
 * @author Andreas T. Meyer-Berg
 */
@SuppressWarnings("serial")
public class MainFrame extends JFrame {
	
	public final VisualisationPanel panel; 
	
	/**
	 * Model of the smart home
	 */
	private Model model;
	private Controller control;
	
	/**
	 * Creates a new Frame for the program, which is the most outer frame of the application
	 * @param m Model which is represented
	 * @param c Controller which handles the user interaction
	 */
	public MainFrame(Model m, Controller c) {
		//Set model and control
		this.model = m;
		this.control = c;
		
		setTitle("Smart Home Network Simulator");
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		getContentPane().setLayout(new BorderLayout(0, 0));
		
		//Set initial size of the frame
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int width = screenSize.width; 
		int height = screenSize.height;
		this.setBounds(width/8, height/8, 6*width/8, 6*height/8);
		this.setMinimumSize(new Dimension(640, 480));
		
		panel = new VisualisationPanel(model, control);
		getContentPane().add(panel, BorderLayout.CENTER);
		c.addObserver(panel);
		//Update model Dimensions to visualisation size
		
		try {
			/**
			 * Icon of the MainFrame UI
			 */	
			String path = "images/SmartHomeNetworkSim_icon.jpeg";
			//this.setIconImage(ImageIO.read(new File("src/main/resources/images/SmartHomeNetworkSim_icon.jpeg")));
			this.setIconImage(Utility.loadFile(path));
		} catch (Exception e) {
			System.err.println("WARNING: Failed to load Icon image in MainFrame.java");
			e.printStackTrace();
		}
		
		JMenuBar menuBar = new JMenuBar();
		super.setJMenuBar(menuBar);	
		//getContentPane().add(menuBar, BorderLayout.NORTH);
		/*
		JMenuItem mntmOptions = new JMenuItem("Options");
		mntmOptions.setHorizontalAlignment(SwingConstants.LEFT);
		menuBar.add(mntmOptions);
		*/
		JMenuItem mntmSimulation = new JMenuItem("Simulation");
		mntmSimulation.setHorizontalAlignment(SwingConstants.LEFT);
		mntmSimulation.addActionListener(a->{
			SimulationConfigurator sim = new SimulationConfigurator(m.getSim());
			sim.setLocationRelativeTo(this);
			sim.setVisible(true);
		});
		menuBar.add(mntmSimulation);
		/*
		JMenuItem mntmOption = new JMenuItem("Options");
		menuBar.add(mntmOption);
		menuBar.setLayout(new FlowLayout(FlowLayout.LEFT));
		*/
		this.setVisible(true);
		//Set Dimension of the model to the JPanel size
		panel.delayedInit();
		
		//Update components if window was maximized
		this.addWindowStateListener(new WindowStateListener() {
			@Override
			public void windowStateChanged(WindowEvent e) {
				if ((e.getNewState() & MainFrame.MAXIMIZED_BOTH) == MainFrame.MAXIMIZED_BOTH){
					revalidate();
					panel.revalidate();
					panel.repaint();
				}
			}
		});
		panel.requestFocusInWindow();
	}
}
